"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBlocklist = exports.removeFromBlocklist = exports.addToBlocklist = exports.setDefaultServiceAccountPath = exports.getServiceAccountPath = void 0;
const fs = require("fs");
const utils_1 = require("./utils");
const Config_1 = require("../types/Config");
const promptTools_1 = require("./promptTools");
const configPath = '../config.json';
/**
 * Get the configuration file as an object.
 * @return The local config in a Config object.
 */
function getConfig() {
    if (!fs.existsSync(configPath))
        fs.writeFileSync(configPath, JSON.stringify({ blocklist: [] }));
    try {
        return (0, utils_1.parseFile)(configPath, Config_1.configDecoder);
    }
    catch (e) {
        return (0, promptTools_1.exitProcess)(1, 'Config file corrupted, please fix it or delete it: config.json');
    }
}
/**
 * Save a config object in the file config.json.
 * @param { Config } config config to save.
 */
function saveConfig(config) {
    fs.writeFileSync(configPath, JSON.stringify(config));
}
/**
 * Get the service account path saved in the config.json.
 * @return Either a service account path or undefined if none can be found in the local config.
 */
function getServiceAccountPath() {
    const config = getConfig();
    return config.serviceAccountPath;
}
exports.getServiceAccountPath = getServiceAccountPath;
/**
 * Set the service account path in the config.json. You can pass it an existing config object if you already instantiated one.
 * @param { string } serviceAccountPath path to the service account.
 * @param { Config | undefined } config already instantiated config object.
 */
function setDefaultServiceAccountPath(serviceAccountPath, config) {
    if (!config)
        config = getConfig();
    config.serviceAccountPath = serviceAccountPath;
    saveConfig(config);
}
exports.setDefaultServiceAccountPath = setDefaultServiceAccountPath;
/**
 * Add projects to the blocklist in config.json.
 * @param { string[] } projectIds list of projects to add to the blocklist.
 * @param { Config | undefined } config already instantiated config object.
 */
function addToBlocklist(projectIds, config) {
    if (!config)
        config = getConfig();
    for (const projectId of projectIds) {
        if (!config.blocklist.includes(projectId))
            config.blocklist.push(projectId);
    }
    saveConfig(config);
}
exports.addToBlocklist = addToBlocklist;
/**
 * Remove projects from the blocklist in config.json.
 * @param { string[] } projectIds list of projects to remove from the blocklist.
 * @param { Config | undefined } config already instantiated config object.
 */
function removeFromBlocklist(projectIds, config) {
    if (!config)
        config = getConfig();
    config.blocklist = config.blocklist.filter((projectId) => !projectIds.includes(projectId));
    saveConfig(config);
}
exports.removeFromBlocklist = removeFromBlocklist;
/**
 * Get the blocklist from config.json.
 * @return The list of blocklisted projects in the local config.
 */
function getBlocklist() {
    const config = getConfig();
    return config.blocklist;
}
exports.getBlocklist = getBlocklist;
//# sourceMappingURL=configTools.js.map