"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportJsonFromRemoteConfig = exports.importJsonToRemoteConfig = exports.copyCollectionAcrossProjects = exports.deleteCollectionsFromFirestore = exports.exportJsonFromFirestore = exports.importJsonToFirestoreWithBatch = exports.importJsonToFirestore = exports.validateCollectionList = exports.collectionsExists = exports.selectManyBetweenExistingCollections = exports.selectOneBetweenExistingCollections = void 0;
const utils_1 = require("./utils");
const promptTools_1 = require("./promptTools");
const fs = require("fs");
const path = require("path");
const fs_1 = require("fs");
/**
 * Ask user to choose one collection of the project.
 * @param { Firestore } db firestore instance.
 */
async function selectOneBetweenExistingCollections(db) {
    const collections = await db.listCollections();
    const collectionsName = collections.map((collection) => collection.id);
    return (0, promptTools_1.promptListSelection)(collectionsName, 'No collection name given, please select one of the followings:');
}
exports.selectOneBetweenExistingCollections = selectOneBetweenExistingCollections;
/**
 * Ask user to choose one or more collections of the project.
 * @param { Firestore } db firestore instance.
 */
async function selectManyBetweenExistingCollections(db) {
    const collections = await db.listCollections();
    const collectionsName = collections.map((collection) => collection.id);
    return (0, promptTools_1.promptCheckbox)(collectionsName, 'Select collection(s) from the followings:');
}
exports.selectManyBetweenExistingCollections = selectManyBetweenExistingCollections;
/**
 * Check if the collections exist in the given project.
 * Returns true if all collections exist in the given project.
 * @param { Firestore } db firestore instance.
 * @param { string[] } collections list of collection names.
 */
async function collectionsExists(db, collections) {
    const actualCollections = await db.listCollections();
    const actualCollectionsName = actualCollections.map((collection) => collection.id);
    return collections.every((collectionName) => actualCollectionsName.includes(collectionName));
}
exports.collectionsExists = collectionsExists;
/**
 * Check the validity of given collections, ask the user to choose one or more if no collection given.
 * If allCollections is true and no collection is given, all the collections of the project will be selected.
 * @return A list of validated collections.
 * @param { Firestore } db firestore instance.
 * @param { boolean } allCollections select all collections of the project.
 * @param { string[] } collectionNames list of the names of the selected collections.
 * @param { string } projectId id of the current project id for logging.
 * @param { boolean } askUserIfInvalid true if you want to ask the user to select collections in case the given ones are not valid.
 */
async function validateCollectionList(db, allCollections, collectionNames, projectId, askUserIfInvalid = true) {
    if (allCollections)
        return db.listCollections().then((l) => l.map((c) => c.id));
    if (!collectionNames || collectionNames.length <= 0)
        return (0, promptTools_1.exitProcess)(1, `No collection found in the project`);
    if (await collectionsExists(db, collectionNames))
        return collectionNames;
    (0, promptTools_1.logError)(`One or more given collection cannot be found in the ${projectId} project`);
    return askUserIfInvalid ? selectManyBetweenExistingCollections(db) : (0, promptTools_1.exitProcess)(1);
}
exports.validateCollectionList = validateCollectionList;
/** Firestore **/
/**
 * Import a JSON file to firestore using.
 * @param { string } jsonPath path to the json to import.
 * @param { Firestore } db firestore instance.
 */
async function importJsonToFirestore(jsonPath, db) {
    const collections = (0, utils_1.parseFile)(jsonPath);
    for (const collectionName in collections) {
        const documents = collections[collectionName];
        if (!documents)
            continue;
        await (0, promptTools_1.executeOperationWithProgressBar)(Object.keys(documents).length, `Importing ${collectionName}`, async (increment) => {
            const operations = [];
            for (const documentName in documents) {
                operations.push(db
                    .collection(collectionName)
                    .doc(documentName)
                    .set(documents[documentName])
                    .then(increment));
            }
            await Promise.all(operations);
        });
    }
}
exports.importJsonToFirestore = importJsonToFirestore;
/**
 * Import a JSON file to firestore using the batch api.
 * @param { string } jsonPath path to the json to import.
 * @param { Firestore } db firestore instance.
 */
async function importJsonToFirestoreWithBatch(jsonPath, db) {
    const collections = (0, utils_1.parseFile)(jsonPath);
    const batch = db.batch();
    for (const collectionName in collections) {
        const documents = collections[collectionName];
        if (!documents)
            continue;
        for (const documentName in documents) {
            batch.set(db.collection(collectionName).doc(documentName), documents[documentName]);
        }
    }
    return batch.commit();
}
exports.importJsonToFirestoreWithBatch = importJsonToFirestoreWithBatch;
/**
 * Export one or more collections as a JSON.
 * @param { string[] } collectionsName names of the collections to export.
 * @param { string } jsonPath path of the file to save.
 * @param { Firestore } db firestore instance.
 */
async function exportJsonFromFirestore(collectionsName, jsonPath, db) {
    await (0, promptTools_1.executeOperationWithProgressBar)(collectionsName.length, 'Exporting collections', async (increment) => {
        fs.mkdirSync(path.dirname(jsonPath), { recursive: true });
        const jsonData = {};
        await Promise.all(collectionsName.map((collectionName) => {
            return db
                .collection(collectionName)
                .get()
                .then((collection) => {
                jsonData[collectionName] = {};
                collection.docs.forEach((doc) => {
                    jsonData[collectionName][doc.id] = doc.data();
                });
                increment();
            });
        }));
        (0, fs_1.writeFileSync)(jsonPath, JSON.stringify(jsonData));
    });
}
exports.exportJsonFromFirestore = exportJsonFromFirestore;
/**
 * Delete one or more collections from firestore.
 * @param { string } collectionsName name of the collections to delete.
 * @param { Firestore } db firestore instance.
 */
async function deleteCollectionsFromFirestore(collectionsName, db) {
    const batch = db.batch();
    for (const collectionName of collectionsName) {
        await db
            .collection(collectionName)
            .listDocuments()
            .then(async (documents) => {
            documents.forEach((doc) => {
                batch.delete(doc);
            });
        });
    }
    return batch.commit();
}
exports.deleteCollectionsFromFirestore = deleteCollectionsFromFirestore;
/**
 * Copy one or more collections from a project to another.
 * @param { string[] } collectionsName name of the collections to copy.
 * @param { Firestore } sourceDb firestore instance of the source project.
 * @param { Firestore } destinationDb firestore instance of the destination project.
 */
async function copyCollectionAcrossProjects(collectionsName, sourceDb, destinationDb) {
    for (const collectionName of collectionsName) {
        const sourceCollection = sourceDb.collection(collectionName);
        const destinationCollection = destinationDb.collection(collectionName);
        const documents = (await sourceCollection.get()).docs;
        await (0, promptTools_1.executeOperationWithProgressBar)(documents.length, `Copying ${collectionName}`, async (increment) => {
            for (const document of documents) {
                await destinationCollection.doc(document.id).set(document.data());
                increment();
            }
        });
    }
}
exports.copyCollectionAcrossProjects = copyCollectionAcrossProjects;
/** Remote Config **/
async function importJsonToRemoteConfig(jsonPath, app) {
    try {
        const template = (0, utils_1.parseFile)(jsonPath);
        const remoteConfig = app.remoteConfig();
        const validatedTemplate = await remoteConfig.validateTemplate(template);
        await remoteConfig.publishTemplate(validatedTemplate);
        return true;
    }
    catch (error) {
        return error;
    }
}
exports.importJsonToRemoteConfig = importJsonToRemoteConfig;
async function exportJsonFromRemoteConfig(jsonPath, app) {
    const remoteConfig = app.remoteConfig();
    await remoteConfig
        .getTemplate()
        .then((template) => {
        const templateString = JSON.stringify(template);
        fs.writeFileSync(jsonPath, templateString);
    })
        .catch(console.error);
}
exports.exportJsonFromRemoteConfig = exportJsonFromRemoteConfig;
//# sourceMappingURL=firebaseTools.js.map