"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeOperationWithProgressBar = exports.getFilenameWithOverwriteValidation = exports.promptBinaryQuestion = exports.promptOpenQuestion = exports.promptCheckbox = exports.promptListSelection = exports.promptCustomValidateOrExit = exports.promptValidateOrExit = exports.promptProjectInfos = exports.exitProcess = exports.logSuccess = exports.logError = void 0;
const chalk = require("chalk");
const inquirer = require("inquirer");
const cli_progress_1 = require("cli-progress");
const fs = require("fs");
/** Log **/
function logError(message) {
    console.log(chalk.red(message));
}
exports.logError = logError;
function logSuccess(message) {
    console.log(chalk.green(message));
}
exports.logSuccess = logSuccess;
function exitProcess(code = 0, message = 'Operation canceled.') {
    logError(message);
    return process.exit(code);
}
exports.exitProcess = exitProcess;
/** Prompt **/
/**
 *
 * @param serviceAccount
 */
function promptProjectInfos(serviceAccount) {
    console.log(`== You are currently working on the project: ${chalk.red(serviceAccount.project_id)}`);
}
exports.promptProjectInfos = promptProjectInfos;
/**
 * Prompt a message and wait for user validation or cancellation. Expected inputs from user are yes or no.
 * Nothing is returned, the process either continues or is exited.
 * @param { string } message message to prompt.
 */
async function promptValidateOrExit(message) {
    await inquirer
        .prompt({
        type: 'confirm',
        name: 'isValid',
        message: message,
    })
        .then((answer) => {
        if (!answer.isValid)
            exitProcess();
    });
}
exports.promptValidateOrExit = promptValidateOrExit;
/**
 * Prompt a message and wait for user validation or cancellation.
 * Expected inputs from user are continueInput or cancelInput.
 * hintMessage is prompted when the user input does not match with the expected ones.
 * Nothing is returned, the process either continues or is exited.
 * @param { string } message message to prompt.
 * @param { string } continueInput expected input to continue process.
 * @param { string } cancelInput expected input to cancel process.
 * @param { string | undefined } hintMessage message prompted if the user does not type an expected input.
 */
async function promptCustomValidateOrExit(message, continueInput, cancelInput, hintMessage = '') {
    await inquirer
        .prompt({
        type: 'input',
        name: 'input',
        message: message,
        validate: (input) => input == continueInput || input == cancelInput ? true : hintMessage,
    })
        .then((answer) => {
        if (answer.input == cancelInput)
            exitProcess();
    });
}
exports.promptCustomValidateOrExit = promptCustomValidateOrExit;
/**
 * Prompt a list within witch the user can select one element.
 * @return The element selected.
 * @param { string[] } choices list of the choices to select from.
 * @param { string } message message prompted to the user.
 */
async function promptListSelection(choices, message) {
    return inquirer
        .prompt({
        name: 'element',
        type: 'list',
        choices: choices,
        message: message,
    })
        .then((answer) => answer.element);
}
exports.promptListSelection = promptListSelection;
/**
 * Prompt a list within witch the user can select one or more elements.
 * @return All the selected elements.
 * @param { string[] } choices list of the choices to select from.
 * @param { string } message message prompted to the user.
 */
async function promptCheckbox(choices, message) {
    return inquirer
        .prompt({
        name: 'list',
        type: 'checkbox',
        choices: choices,
        message: message,
    })
        .then((answer) => answer.list);
}
exports.promptCheckbox = promptCheckbox;
/**
 * Prompt a question to the user and wait for an input. The input can be validated with the validation function.
 * @return The user input.
 * @param { string } message message prompted to the user.
 * @param { (string) => (string | boolean) | undefined } validate function that validates user input.
 */
async function promptOpenQuestion(message, validate = () => true) {
    return inquirer
        .prompt({
        name: 'input',
        type: 'input',
        message: message,
        validate: validate,
    })
        .then((answer) => answer.input);
}
exports.promptOpenQuestion = promptOpenQuestion;
/**
 * Prompt a question to the user and wait for a binary response. The input can be validated with the validation function.
 * @return The result of the validate function applied on the user input.
 * @param { string } message message prompted to the user.
 * @param { (string) => (string | boolean) | undefined } validate function that validates user input.
 */
async function promptBinaryQuestion(message, validate = () => true) {
    return inquirer
        .prompt({
        name: 'isValid',
        type: 'confirm',
        message: message,
        validate: validate,
    })
        .then((answer) => answer.isValid);
}
exports.promptBinaryQuestion = promptBinaryQuestion;
/**
 * Generate a filename if the  `path` already exists and the user doesn't want to overwrite it.
 * @return { string } Generated filename.
 * @param { string | undefined } path path with filename to check.
 * @param { boolean | undefined } overwrite force to overwrite the file.
 * @param { string | undefined } defaultPrefix filename prefix.
 * @param { boolean } askUserIfOverwrite true if you want to ask the user if he wants to overwrite.
 */
async function getFilenameWithOverwriteValidation(path, overwrite, defaultPrefix = '', askUserIfOverwrite = true) {
    let filename = path !== null && path !== void 0 ? path : `${defaultPrefix}-${Date.now()}.json`;
    if (fs.existsSync(filename) &&
        !overwrite &&
        !(askUserIfOverwrite
            ? await promptBinaryQuestion(`File already exists: ${filename}.\nWould you like to overwrite it ?`)
            : exitProcess(1, `File already exists: ${filename}.`))) {
        let i = 1;
        while (fs.existsSync(`${filename}-${i}`)) {
            ++i;
        }
        filename = `${filename}-${i}`;
    }
    return filename;
}
exports.getFilenameWithOverwriteValidation = getFilenameWithOverwriteValidation;
/** Progress Bar **/
function getProgressBar(size, prefix = '') {
    const progressBar = new cli_progress_1.SingleBar({
        format: chalk.bgGreen(`${prefix} |{bar}| {percentage}%`),
    }, cli_progress_1.Presets.shades_classic);
    progressBar.start(size, 0);
    return progressBar;
}
async function executeOperationWithProgressBar(size, prefix = '', executeOperation) {
    const progressBar = getProgressBar(size, prefix);
    await executeOperation(() => progressBar.increment());
    progressBar.stop();
}
exports.executeOperationWithProgressBar = executeOperationWithProgressBar;
//# sourceMappingURL=promptTools.js.map