import * as admin from 'firebase-admin';
import { ServiceAccount } from '../types/ServiceAccount';
/**
 * Check if the given file path is a valid service account file.
 * @return Either true if valid or an error message if invalid.
 * @param { string | undefined } serviceAccountPath file path to check.
 * @param { boolean } printError true if you want to print the errors.
 */
export declare function isValidServiceAccountPath(serviceAccountPath: string | undefined, printError?: boolean): string | boolean;
/**
 * Get a service account file path with the user input and a path validation.
 * @return A valid service account file path.
 * @param { string | undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
export declare function getServiceAccountPathWithUserInput(customMessage?: string): Promise<string>;
/**
 * Get a service account object with user input and a projectId validation from user.
 * @return A valid service account object.
 * @param { string | undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
export declare function getServiceAccountWithUserInput(customMessage?: string): Promise<ServiceAccount>;
/**
 * Check the validity and parse a service account file path. If not valid, the user will be asked to input a new path.
 * @return A valid service account object.
 * @param { string } serviceAccountPath path of the service account file to check and parse.
 * @param { string| undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 * @param { boolean } askUserIfInvalid true if you want to ask the user for a service account path in case the given one is not valid.
 */
export declare function validateAndParseServiceAccountPath(serviceAccountPath: string, askUserIfInvalid?: boolean, customMessage?: string): Promise<ServiceAccount>;
/**
 * Get a service account object using config file or a user input if no config. The user input is saved in the config file.
 * @return A valid service account object.
 * @param { string| undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
export declare function getServiceAccountWithConfigOrUserInput(customMessage?: string): Promise<ServiceAccount>;
/**
 * Get the firebase app corresponding to the given service account object.
 * @return A firebase app object.
 * @param { ServiceAccount } serviceAccount a valid service account object.
 * @param { string | undefined } appName the name of the app (used only to init multiple apps).
 */
export declare function getFirebaseApp(serviceAccount: ServiceAccount, appName?: string): Promise<admin.app.App>;
