"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFirebaseApp = exports.getServiceAccountWithConfigOrUserInput = exports.validateAndParseServiceAccountPath = exports.getServiceAccountWithUserInput = exports.getServiceAccountPathWithUserInput = exports.isValidServiceAccountPath = void 0;
const fs = require("fs");
const admin = require("firebase-admin");
const path = require("path");
const utils_1 = require("./utils");
const ServiceAccount_1 = require("../types/ServiceAccount");
var cert = admin.credential.cert;
const configTools_1 = require("./configTools");
const promptTools_1 = require("./promptTools");
/**
 * Check if the given file path is a valid service account file.
 * @return Either true if valid or an error message if invalid.
 * @param { string | undefined } serviceAccountPath file path to check.
 * @param { boolean } printError true if you want to print the errors.
 */
function isValidServiceAccountPath(serviceAccountPath, printError = false) {
    if (!serviceAccountPath) {
        if (printError)
            (0, promptTools_1.logError)('No service account path specified');
        return 'No service account path specified';
    }
    if (!fs.existsSync(serviceAccountPath)) {
        if (printError)
            (0, promptTools_1.logError)(`File not found: ${serviceAccountPath}`);
        return `File not found: ${serviceAccountPath}`;
    }
    if (path.extname(serviceAccountPath) != '.json') {
        if (printError)
            (0, promptTools_1.logError)('Invalid service account file');
        return 'Invalid service account file';
    }
    try {
        const serviceAccount = (0, utils_1.parseFile)(serviceAccountPath, ServiceAccount_1.serviceAccountDecoder);
        if ((0, configTools_1.getBlocklist)().includes(serviceAccount.project_id)) {
            if (printError)
                (0, promptTools_1.logError)(`The project '${serviceAccount.project_id}' is blocklisted`);
            return `The project '${serviceAccount.project_id}' is blocklisted`;
        }
        return true;
    }
    catch (e) {
        if (printError)
            (0, promptTools_1.logError)('Invalid service account file: ' + e);
        return 'Invalid service account file';
    }
}
exports.isValidServiceAccountPath = isValidServiceAccountPath;
/**
 * Get a service account file path with the user input and a path validation.
 * @return A valid service account file path.
 * @param { string | undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
async function getServiceAccountPathWithUserInput(customMessage) {
    return await (0, promptTools_1.promptOpenQuestion)(customMessage !== null && customMessage !== void 0 ? customMessage : 'What is the path to your firebase project service account ?', (input) => isValidServiceAccountPath(input));
}
exports.getServiceAccountPathWithUserInput = getServiceAccountPathWithUserInput;
/**
 * Get a service account object with user input and a projectId validation from user.
 * @return A valid service account object.
 * @param { string | undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
async function getServiceAccountWithUserInput(customMessage) {
    const serviceAccountPath = await getServiceAccountPathWithUserInput(customMessage);
    let serviceAccount = (0, utils_1.parseFile)(serviceAccountPath, ServiceAccount_1.serviceAccountDecoder);
    return (await (0, promptTools_1.promptBinaryQuestion)(`Do you want to use the '${serviceAccount.project_id}' project ?`))
        ? serviceAccount
        : await getServiceAccountWithUserInput(customMessage);
}
exports.getServiceAccountWithUserInput = getServiceAccountWithUserInput;
/**
 * Check the validity and parse a service account file path. If not valid, the user will be asked to input a new path.
 * @return A valid service account object.
 * @param { string } serviceAccountPath path of the service account file to check and parse.
 * @param { string| undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 * @param { boolean } askUserIfInvalid true if you want to ask the user for a service account path in case the given one is not valid.
 */
async function validateAndParseServiceAccountPath(serviceAccountPath, askUserIfInvalid = true, customMessage) {
    const isValid = isValidServiceAccountPath(serviceAccountPath, true);
    return isValid != true
        ? askUserIfInvalid
            ? getServiceAccountWithUserInput(customMessage)
            : (0, promptTools_1.exitProcess)(1, `${isValid}`)
        : (0, utils_1.parseFile)(serviceAccountPath, ServiceAccount_1.serviceAccountDecoder);
}
exports.validateAndParseServiceAccountPath = validateAndParseServiceAccountPath;
/**
 * Get a service account object using config file or a user input if no config. The user input is saved in the config file.
 * @return A valid service account object.
 * @param { string| undefined } customMessage message prompted to the user. If not given, a default one will be prompted.
 */
async function getServiceAccountWithConfigOrUserInput(customMessage) {
    let serviceAccountPath = (0, configTools_1.getServiceAccountPath)();
    if (isValidServiceAccountPath(serviceAccountPath, true) != true) {
        serviceAccountPath = await getServiceAccountPathWithUserInput(customMessage);
        (0, configTools_1.setDefaultServiceAccountPath)(serviceAccountPath);
    }
    return (0, utils_1.parseFile)(serviceAccountPath, ServiceAccount_1.serviceAccountDecoder);
}
exports.getServiceAccountWithConfigOrUserInput = getServiceAccountWithConfigOrUserInput;
/**
 * Get the firebase app corresponding to the given service account object.
 * @return A firebase app object.
 * @param { ServiceAccount } serviceAccount a valid service account object.
 * @param { string | undefined } appName the name of the app (used only to init multiple apps).
 */
async function getFirebaseApp(serviceAccount, appName) {
    return admin.initializeApp({
        credential: cert({
            projectId: serviceAccount.project_id,
            clientEmail: serviceAccount.client_email,
            privateKey: serviceAccount.private_key,
        }),
    }, appName);
}
exports.getFirebaseApp = getFirebaseApp;
//# sourceMappingURL=serviceAccountTools.js.map